import type { IAutorunOptions, IReactionDisposer, IReactionOptions, IReactionPublic } from 'mobx';
import type { OmitValueOfKey, OmitValueWithType } from 'tslang';
import type { IHistory } from '../history';
import type { Router } from '../router';
import type { NextRouteMatch } from './next-route-match';
import type { GeneralParamDict, RouteMatchSharedOptions } from './route-match-shared';
import { RouteMatchShared } from './route-match-shared';
export declare const ROUTE_SERVICE_ENTER_DATA_SYMBOL: unique symbol;
export declare const ROUTE_SERVICE_UPDATE_DATA_SYMBOL: unique symbol;
export interface RouteUpdateContext {
    descendants: boolean;
}
/**
 * Route before enter callback.
 * @return Return `true` or `undefined` to do nothing; return `false` to revert
 * this history change; return full path to redirect.
 */
export type RouteBeforeEnterCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next']) => Promise<boolean | void> | boolean | void;
export type ServiceBeforeEnterCallback<TRouteMatch extends RouteMatch = RouteMatch, TEnterData extends object | void = object | void> = (next: TRouteMatch['$next']) => Promise<boolean | TEnterData> | boolean | TEnterData;
/**
 * Route before update callback.
 * @return Return `true` or `undefined` to do nothing; return `false` to revert
 * this history change; return full path to redirect.
 */
export type RouteBeforeUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next'], context: RouteUpdateContext) => Promise<boolean | void> | boolean | void;
export type ServiceBeforeUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch, TUpdateData extends object | void = object | void> = (next: TRouteMatch['$next'], context: RouteUpdateContext) => Promise<boolean | TUpdateData> | boolean | TUpdateData;
export interface RouteBeforeUpdateOptions {
    traceDescendants: boolean;
}
export interface RouteBeforeUpdateEntry<TRouteMatch extends RouteMatch = RouteMatch> {
    callback: RouteBeforeUpdateCallback<TRouteMatch>;
    options?: RouteBeforeUpdateOptions;
}
/**
 * Route before leave callback.
 * @return Return `true` or `undefined` to do nothing; return `false` to revert
 * this history change.
 */
export type RouteBeforeLeaveCallback = () => Promise<boolean | void> | boolean | void;
export type RouteWillEnterCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next']) => Promise<void> | void;
export type ServiceWillEnterCallback<TRouteMatch extends RouteMatch = RouteMatch, TEnterData extends object | void = object | void> = (next: TRouteMatch['$next'], data: TEnterData) => Promise<void> | void;
export type RouteWillUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next'], context: RouteUpdateContext) => Promise<void> | void;
export type ServiceWillUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch, TUpdateData extends object | void = object | void> = (next: TRouteMatch['$next'], context: RouteUpdateContext, data: TUpdateData) => Promise<void> | void;
export interface RouteWillUpdateOptions {
    traceDescendants: boolean;
}
export interface RouteWillUpdateEntry<TRouteMatch extends RouteMatch = RouteMatch> {
    callback: RouteWillUpdateCallback<TRouteMatch>;
    options?: RouteWillUpdateOptions;
}
export type RouteWillLeaveCallback = () => Promise<void> | void;
export type RouteEnterCallback = () => void;
export type ServiceEnterCallback<TEnterData extends object | void = object | void> = (data: TEnterData) => void;
export type RouteUpdateCallback = (context: RouteUpdateContext) => void;
export type ServiceUpdateCallback<TUpdateData extends object | void> = (context: RouteUpdateContext, data: TUpdateData) => void;
export interface RouteUpdateOptions {
    traceDescendants: boolean;
}
export interface RouteUpdateEntry {
    callback: RouteUpdateCallback;
    options?: RouteUpdateOptions;
}
export type RouteLeaveCallback = () => void;
export type RouteAfterEnterCallback = () => void;
/**
 * Route after update callback.
 */
export type RouteAfterUpdateCallback = (context: RouteUpdateContext) => void;
export interface RouteAfterUpdateOptions {
    traceDescendants: boolean;
}
export interface RouteAfterUpdateEntry {
    options?: RouteAfterUpdateOptions;
    callback: RouteAfterUpdateCallback;
}
export type RouteAfterLeaveCallback = () => void;
export type RouteReactiveDisposer = IReactionDisposer;
export type RouteAutorunView = (reaction: IReactionPublic) => void;
export type RouteAutorunOptions = IAutorunOptions;
export type RouteReactionExpression<T> = (reaction: IReactionPublic) => T;
export type RouteReactionEffect<T> = (value: T, previousValue: T, reaction: IReactionPublic) => void;
export type RouteReactionOptions<T, FireImmediately extends boolean = boolean> = IReactionOptions<T, FireImmediately> | undefined;
export type RouteHookRemovalCallback = () => void;
export type RouteBeforeEnterOrUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next']) => Promise<boolean | void> | boolean | void;
export type RouteWillEnterOrUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next']) => Promise<void> | void;
export type RouteAfterEnterOrUpdateCallback = () => void;
export type RouteServiceFactory<TRouteMatch extends RouteMatch> = (match: TRouteMatch) => IRouteService<TRouteMatch> | Promise<IRouteService<TRouteMatch>>;
export type IRouteService<TRouteMatch extends RouteMatch = RouteMatch, TEnterData extends object | void = object | void, TUpdateData extends object | void = object | void> = {
    beforeEnter?: ServiceBeforeEnterCallback<TRouteMatch, TEnterData>;
    willEnter?: ServiceWillEnterCallback<TRouteMatch, TEnterData>;
    enter?: ServiceEnterCallback<TEnterData>;
    afterEnter?: RouteAfterEnterCallback;
    beforeUpdate?: ServiceBeforeUpdateCallback<TRouteMatch, TUpdateData>;
    willUpdate?: ServiceWillUpdateCallback<TRouteMatch, TUpdateData>;
    update?: ServiceUpdateCallback<TUpdateData>;
    afterUpdate?: RouteAfterUpdateCallback;
    beforeLeave?: RouteBeforeLeaveCallback;
    willLeave?: RouteWillLeaveCallback;
    leave?: RouteLeaveCallback;
    afterLeave?: RouteAfterLeaveCallback;
} & RouteServiceExtension<TRouteMatch>;
export type RouteServiceExtension<TRouteMatch extends RouteMatch> = OmitValueWithType<OmitValueOfKey<TRouteMatch, keyof RouteMatch>, RouteMatch, false>;
export interface RouteMatchResult {
    matched: boolean;
    exactlyMatched: boolean;
    rest: string;
}
export interface RouteMatchParallelOptions<TGroupName extends string> {
    groups?: TGroupName[];
    matches?: RouteMatch[];
}
export interface RouteMatchEntry {
    match: RouteMatch;
    exact: boolean;
    segment: string;
    rest: string;
}
export interface RouteSourceQuery {
    id: string | symbol | true;
    value: string;
}
export interface RouteSource {
    groupToMatchToMatchEntryMapMap: Map<string | undefined, Map<RouteMatch, RouteMatchEntry>>;
    queryMap: Map<string, RouteSourceQuery>;
    pathMap: Map<string | undefined, string>;
}
export interface RouteMatchOptions extends RouteMatchSharedOptions {
    exact: boolean | string;
}
export declare class RouteMatch<TParamDict extends GeneralParamDict = GeneralParamDict, TNextRouteMatch extends NextRouteMatch<TParamDict> = NextRouteMatch<TParamDict>, TSpecificGroupName extends string | undefined = string | undefined, TGroupName extends string = string, TMetadata extends object = object> extends RouteMatchShared<TParamDict, TSpecificGroupName, TGroupName, TMetadata> {
    readonly $parent: RouteMatch | undefined;
    readonly $next: TNextRouteMatch;
    constructor(name: string, router: Router<TGroupName>, source: RouteSource, parent: RouteMatch | undefined, extension: object | undefined, history: IHistory, { exact, ...sharedOptions }: RouteMatchOptions);
    $beforeEnter(callback: RouteBeforeEnterCallback<this>): RouteHookRemovalCallback;
    $beforeUpdate(callback: RouteBeforeUpdateCallback<this>, options?: RouteBeforeUpdateOptions): RouteHookRemovalCallback;
    $beforeLeave(callback: RouteBeforeLeaveCallback): RouteHookRemovalCallback;
    $willEnter(callback: RouteWillEnterCallback): RouteHookRemovalCallback;
    $willUpdate(callback: RouteWillUpdateCallback, options?: RouteWillUpdateOptions): RouteHookRemovalCallback;
    $willLeave(callback: RouteWillLeaveCallback): RouteHookRemovalCallback;
    $enter(callback: RouteEnterCallback): RouteHookRemovalCallback;
    $update(callback: RouteUpdateCallback, options?: RouteUpdateOptions): RouteHookRemovalCallback;
    $leave(callback: RouteLeaveCallback): RouteHookRemovalCallback;
    $afterEnter(callback: RouteAfterEnterCallback): RouteHookRemovalCallback;
    $afterUpdate(callback: RouteAfterUpdateCallback, options?: RouteAfterUpdateOptions): RouteHookRemovalCallback;
    $afterLeave(callback: RouteAfterLeaveCallback): RouteHookRemovalCallback;
    $autorun(view: RouteAutorunView, options?: RouteAutorunOptions): RouteHookRemovalCallback;
    $reaction<T>(expression: RouteReactionExpression<T>, effect: RouteReactionEffect<T>, options?: RouteReactionOptions<T>): RouteHookRemovalCallback;
    $beforeEnterOrUpdate(callback: RouteBeforeEnterOrUpdateCallback<this>, beforeUpdateOptions?: RouteBeforeUpdateOptions): RouteHookRemovalCallback;
    $willEnterOrUpdate(callback: RouteWillEnterOrUpdateCallback<this>, willUpdateOptions?: RouteWillUpdateOptions): RouteHookRemovalCallback;
    $afterEnterOrUpdate(callback: RouteAfterEnterOrUpdateCallback, afterUpdateOptions?: RouteAfterUpdateOptions): RouteHookRemovalCallback;
    $service(factory: RouteServiceFactory<this>): this;
    $parallel(options: RouteMatchParallelOptions<TGroupName>): void;
    $match(path: string): RouteMatchResult;
    static SEGMENT: RegExp;
    static REST: RegExp;
}
